@@ -61,66 +61,50 @@ def migrate_repositories(plan):
     )
     with ProgressReport(**progress_data) as pb:
         for plugin in plan.get_plugin_plans():
+            # all pulp2 repos in current plan were already migrated, no need to proceed
+            not_migrated_repos = Pulp2Repository.objects.filter(
+                is_migrated=False,
+                not_in_plan=False,
+                pulp2_repo_type=plugin.type
+            )
+            if not not_migrated_repos.exists():
+                continue
 
-            # TODO: Should we filter out repos which have is_migrated=True?
             pulp2repos_qs = Pulp2Repository.objects.filter(
                 pulp3_repository_version=None,
                 not_in_plan=False,
                 pulp2_repo_type=plugin.type,
             )
             repos_to_create = plugin.get_repo_creation_setup()
+            pb.total += len(repos_to_create)
+            pb.save()
 
-            # no specific migration plan for repositories
-            if not repos_to_create:
-                pb.total += pulp2repos_qs.count()
-                pb.save()
+            for pulp3_repo_name in repos_to_create:
+                try:
+                    pulp2repo = pulp2repos_qs.get(pulp2_repo_id=pulp3_repo_name)
+                except Pulp2Repository.DoesNotExist:
+                    description = pulp3_repo_name
+                else:
+                    description = pulp2repo.pulp2_description
+                repository_class = plugin.migrator.pulp3_repository
+                repo, created = repository_class.objects.get_or_create(
+                    name=pulp3_repo_name,
+                    defaults={'description': description})
 
-                for pulp2repo in pulp2repos_qs:
-                    pulp3_repo_name = pulp2repo.pulp2_repo_id
-                    repository_class = plugin.migrator.pulp3_repository
-                    repo, created = repository_class.objects.get_or_create(
-                        name=pulp3_repo_name,
-                        defaults={'description': pulp2repo.pulp2_description})
-                    if not pulp2repo.pulp3_repository:
-                        pulp2repo.pulp3_repository = repo
-                        pulp2repo.save()
-                    if created:
-                        pb.increment()
-                    else:
-                        pb.total -= 1
-                        pb.save()
+                pulp2_repo_ids = []
+                repo_version_setup = repos_to_create[pulp3_repo_name].get('repository_versions')
+                for repo_version in repo_version_setup:
+                    pulp2_repo_ids.append(repo_version['repo_id'])
+                pulp2repos_qs = Pulp2Repository.objects.filter(
+                    pulp2_repo_id__in=pulp2_repo_ids, pulp3_repository__isnull=True
+                )
+                pulp2repos_qs.update(pulp3_repository=repo)
 
-            # specific migration plan for repositories
-            else:
-                pb.total += len(repos_to_create)
-                pb.save()
-
-                for pulp3_repo_name in repos_to_create:
-                    try:
-                        pulp2repo = pulp2repos_qs.get(pulp2_repo_id=pulp3_repo_name)
-                    except Pulp2Repository.DoesNotExist:
-                        description = pulp3_repo_name
-                    else:
-                        description = pulp2repo.pulp2_description
-                    repository_class = plugin.migrator.pulp3_repository
-                    repo, created = repository_class.objects.get_or_create(
-                        name=pulp3_repo_name,
-                        defaults={'description': description})
-
-                    pulp2_repo_ids = []
-                    repo_version_setup = repos_to_create[pulp3_repo_name].get('repository_versions')
-                    for repo_version in repo_version_setup:
-                        pulp2_repo_ids.append(repo_version['repo_id'])
-                    pulp2repos_qs = Pulp2Repository.objects.filter(pulp2_repo_id__in=pulp2_repo_ids)
-                    for pulp2repo in pulp2repos_qs:
-                        if not pulp2repo.pulp3_repository:
-                            pulp2repo.pulp3_repository = repo
-                            pulp2repo.save()
-                    if created:
-                        pb.increment()
-                    else:
-                        pb.total -= 1
-                        pb.save()
+                if created:
+                    pb.increment()
+                else:
+                    pb.total -= 1
+                    pb.save()
 
 
 def migrate_importers(plan):
@@ -207,7 +191,7 @@ def complex_repo_migration(plugin, pulp3_repo_setup, repo_name):
             # it's possible to have a random order of the repo versions (after migration
             # re-run, a repo can be changed in pulp 2 and it might not be for the last
             # repo version)
-            create_repo_version(plugin.migrator, progress_rv, pulp2_repo, pulp3_remote)
+            create_repo_version(progress_rv, pulp2_repo, pulp3_remote)
 
     for pulp2_repo_info in repo_versions_setup:
         # find pulp2repo by id
@@ -258,6 +242,22 @@ def create_repoversions_publications_distributions(plan, parallel=True):
         parallel (bool): If True, attempt to migrate things in parallel where possible.
     """
     for plugin in plan.get_plugin_plans():
+        # verify whether all pulp2 repos and distributors have been migrated
+        not_migrated_repos = Pulp2Repository.objects.filter(
+            is_migrated=False,
+            not_in_plan=False,
+            pulp2_repo_type=plugin.type)
+        not_migrated_dists = Pulp2Distributor.objects.filter(
+            is_migrated=False,
+            not_in_plan=False,
+            pulp2_type_id__in=plugin.migrator.distributor_migrators.keys())
+        # no need to proceed - everything is migrated
+        if not not_migrated_repos and not not_migrated_dists:
+            continue
+        not_migrated_repo_ids = not_migrated_repos.values_list('pulp2_repo_id', flat=True)
+        not_migrated_repo_ids_dists = not_migrated_dists.values_list('pulp2_repo_id', flat=True)
+        repos_ids_to_check = set(not_migrated_repo_ids).union(not_migrated_repo_ids_dists)
+
         pulp3_repo_setup = plugin.get_repo_creation_setup()
 
         repo_ver_to_create = 0
@@ -266,22 +266,36 @@ def create_repoversions_publications_distributions(plan, parallel=True):
         if parallel:
             for repo_name in pulp3_repo_setup:
                 repo_versions = pulp3_repo_setup[repo_name]['repository_versions']
-                repo_ver_to_create += len(repo_versions)
+                needs_a_task = False
                 for repo_ver in repo_versions:
-                    dist_to_create += len(repo_ver['dist_repo_ids'])
-                repo = Repository.objects.get(name=repo_name).cast()
-                task_args = [plugin, pulp3_repo_setup, repo_name]
-                enqueue_with_reservation(
-                    complex_repo_migration,
-                    [repo],
-                    args=task_args,
-                    task_group=TaskGroup.current()
-                )
+                    repos = set(repo_ver['dist_repo_ids'] + [repo_ver['repo_id']])
+                    # check whether any resources are not migrated and need a task
+                    if repos.intersection(repos_ids_to_check):
+                        needs_a_task = True
+                        dist_to_create += len(repo_ver['dist_repo_ids'])
+                if needs_a_task:
+                    repo_ver_to_create += len(repo_versions)
+                    repo = Repository.objects.get(name=repo_name).cast()
+                    task_args = [plugin, pulp3_repo_setup, repo_name]
+                    enqueue_with_reservation(
+                        complex_repo_migration,
+                        [repo],
+                        args=task_args,
+                        task_group=TaskGroup.current()
+                    )
         else:
             # Serial (non-parallel)
             for repo_name in pulp3_repo_setup:
-                task_args = [plugin, pulp3_repo_setup, repo_name]
-                complex_repo_migration(*task_args)
+                repo_versions = pulp3_repo_setup[repo_name]['repository_versions']
+                needs_a_task = False
+                for repo_ver in repo_versions:
+                    repos = set(repo_ver['dist_repo_ids'] + [repo_ver['repo_id']])
+                    # check whether any resources are not migrated and need a task
+                    if repos.intersection(repos_ids_to_check):
+                        needs_a_task = True
+                if needs_a_task:
+                    task_args = [plugin, pulp3_repo_setup, repo_name]
+                    complex_repo_migration(*task_args)
 
         task_group = TaskGroup.current()
         progress_rv = task_group.group_progress_reports.filter(code='create.repo_version')
@@ -292,7 +306,7 @@ def create_repoversions_publications_distributions(plan, parallel=True):
         progress_dist.update(total=F('total') + dist_to_create)
 
 
-def create_repo_version(migrator, progress_rv, pulp2_repo, pulp3_remote=None):
+def create_repo_version(progress_rv, pulp2_repo, pulp3_remote=None):
     """
     Create a repo version based on a pulp2 repository.
 
@@ -300,7 +314,6 @@ def create_repo_version(migrator, progress_rv, pulp2_repo, pulp3_remote=None):
     being changed itself, re-set it here for every repo.
 
     Args:
-        migrator: migrator to use, provides repo type information
         progress_rv: GroupProgressReport queryset for repo_version creation
         pulp2_repo(Pulp2Repository): a pre-migrated repository to create a repo version for
         pulp3_remote(remote): a pulp3 remote
@@ -308,11 +321,9 @@ def create_repo_version(migrator, progress_rv, pulp2_repo, pulp3_remote=None):
 
     if pulp3_remote:
         pulp2_repo.pulp3_repository_remote = pulp3_remote
-    # pulp2importer might not be migrated, e.g. config was empty
-    elif hasattr(pulp2_repo, 'pulp2importer'):
-        pulp2_repo.pulp3_repository_remote = pulp2_repo.pulp2importer.pulp3_remote
-    if pulp2_repo.is_migrated:
         pulp2_repo.save()
+
+    if pulp2_repo.is_migrated:
         progress_rv.update(total=F('total') - 1)
         return
 
