@@ -23,6 +23,7 @@ from pulp_2to3_migration.app.models import (
     Pulp2LazyCatalog,
     Pulp2RepoContent,
     Pulp2Repository,
+    RepoSetup,
 )
 from pulp_2to3_migration.pulp2.base import (
     Distributor,
@@ -76,7 +77,7 @@ def pre_migrate_content_type(content_model, mutable_type, lazy_type, premigrate_
         content_model: Models for content which is being migrated.
         mutable_type: Boolean that indicates whether the content type is mutable.
     """
-    batch_size = 500
+    batch_size = 100
     pulp2content = []
     pulp2mutatedcontent = []
     content_type = content_model.pulp2.TYPE_ID
@@ -178,7 +179,7 @@ def pre_migrate_content_type(content_model, mutable_type, lazy_type, premigrate_
                     pulp2_last_updated=record._last_updated,
                     pulp2_storage_path=record._storage_path,
                     downloaded=downloaded,
-                    pulp2_repo=relation.pulp2_repository
+                    pulp2_repo=relation.pulp2_repository,
                 )
                 _logger.debug(
                     'Add content item to the list to migrate: {item}'.format(item=item))
@@ -210,6 +211,25 @@ def pre_migrate_content_type(content_model, mutable_type, lazy_type, premigrate_
             )
             pulp2content_batch = Pulp2Content.objects.bulk_create(pulp2content,
                                                                   ignore_conflicts=True)
+
+            # bulk_create(ignore_conflicts=True) hands back the same item-set we passed in,
+            # *even if* it decided to update an existing db-record rather than creating a new
+            # one with the passed-in PK. As a result, we can't trust pulp2content_batch to
+            # have the 'right' PKs (i.e., the in-memory p2content_batch doesn't match the
+            # db-reality). This causes the pre_migrate_content_detail() below to fail as it
+            # attempts to create detail-records for the Pulp2Content records it's been handed.
+            # THEREFORE - we need to find the 'real' IDs of everything in p2content-batch based
+            # on its uniqueness-fields and update the in-memory list with them.
+            for p2c in pulp2content_batch:
+                filter_q = Q(
+                    pulp2_content_type_id=content_type,
+                    pulp2_id=p2c.pulp2_id,
+                    pulp2_repo=p2c.pulp2_repo,
+                    pulp2_subid=p2c.pulp2_subid,
+                )
+                p2c_db = Pulp2Content.objects.get(filter_q)
+                p2c.pulp_id = p2c_db.pulp_id
+
             content_saved = len(pulp2content_batch) - existing_count
             pulp2content_pb.done += content_saved
             pulp2content_pb.save()
@@ -366,18 +386,14 @@ def pre_migrate_all_without_content(plan):
                                                 'last_unit_added',
                                                 'last_unit_removed',
                                                 'description'):
-                repo = None
                 repo_id = repo_data.repo_id
                 with transaction.atomic():
-                    if not repos or repos and repo_id in repos:
-                        repo = pre_migrate_repo(repo_data, plan.repo_id_to_type)
-                    # do not pre-migrate importers/distributors in case of special repo setup
-                    # and no importers/distributors were specified in the MP
-                    if not repos or repos and importers_repos:
-                        pre_migrate_importer(repo_id, importers_repos, importer_types, repo)
-                    if not repos or repos and distributors_repos:
-                        pre_migrate_distributor(
-                            repo_id, distributors_repos, distributor_migrators, repo)
+                    if repo_id in repos:
+                        pre_migrate_repo(repo_data, plan.repo_id_to_type)
+                    if repo_id in importers_repos:
+                        pre_migrate_importer(repo_id, importer_types)
+                    if repo_id in distributors_repos:
+                        pre_migrate_distributor(repo_id, distributor_migrators)
                     pb.increment()
 
 
@@ -432,26 +448,19 @@ def pre_migrate_repo(record, repo_id_to_type):
     return repo
 
 
-def pre_migrate_importer(repo_id, importers, importer_types, repo=None):
+def pre_migrate_importer(repo_id, importer_types):
     """
     Pre-migrate a pulp 2 importer.
 
     Args:
         repo_id(str): An id of a pulp 2 repository which importer should be migrated
-        importers(list): A list of importers which are expected to be migrated. If empty,
-                         all are migrated.
         importer_types(list): a list of supported importer types
-        repo(Pulp2Repository): A pre-migrated pulp 2 repository for this importer
     """
     mongo_importer_q = mongo_Q(repo_id=repo_id, importer_type_id__in=importer_types)
 
     # importers with empty config are not needed - nothing to migrate
     mongo_importer_q &= mongo_Q(config__exists=True) & mongo_Q(config__ne={})
 
-    # in case only certain importers are specified in the migration plan
-    if importers:
-        mongo_importer_q &= mongo_Q(repo_id__in=importers)
-
     mongo_importer_qs = Importer.objects(mongo_importer_q)
     if not mongo_importer_qs:
         # Either the importer no longer exists in Pulp2,
@@ -480,7 +489,6 @@ def pre_migrate_importer(repo_id, importers, importer_types, repo=None):
         defaults={'pulp2_type_id': importer_data.importer_type_id,
                   'pulp2_last_updated': last_updated,
                   'pulp2_config': importer_data.config,
-                  'pulp2_repository': repo,
                   'pulp2_repo_id': repo_id,
                   'is_migrated': False})
 
@@ -501,25 +509,18 @@ def pre_migrate_importer(repo_id, importers, importer_types, repo=None):
         importer.save()
 
 
-def pre_migrate_distributor(repo_id, distributors, distributor_migrators, repo=None):
+def pre_migrate_distributor(repo_id, distributor_migrators):
     """
     Pre-migrate a pulp 2 distributor.
 
     Args:
         repo_id(str): An id of a pulp 2 repository which distributor should be migrated
-        distributors(list): A list of distributors which are expected to be migrated. If empty,
-                            all are migrated.
         distributor_migrators(dict): supported distributor types and their models for migration
-        repo(Pulp2Repository): A pre-migrated pulp 2 repository for this distributor
     """
     distributor_types = list(distributor_migrators.keys())
     mongo_distributor_q = mongo_Q(repo_id=repo_id,
                                   distributor_type_id__in=distributor_types)
 
-    # in case only certain distributors are specified in the migration plan
-    if distributors:
-        mongo_distributor_q &= mongo_Q(repo_id__in=distributors)
-
     mongo_distributor_qs = Distributor.objects(mongo_distributor_q)
     if not mongo_distributor_qs:
         # Either the distributor no longer exists in Pulp2,
@@ -540,12 +541,6 @@ def pre_migrate_distributor(repo_id, distributors, distributor_migrators, repo=N
                       'pulp2_repo_id': repo_id,
                       'is_migrated': False})
 
-        # this is the case for the simple plan
-        # add native distributor to the repo
-        # this will go away with the simple-complex plan conversion work
-        if not distributors and repo:
-            repo.pulp2_dists.add(distributor)
-
         if not created:
             # if it was marked as such because it was not present in the migration plan
             distributor.not_in_plan = False
@@ -622,15 +617,15 @@ def handle_outdated_resources(plan):
     Args:
         plan(MigrationPlan): A Migration Plan
     """
+    RepoSetup.mark_changed_relations()
+    RepoSetup.finalize()
+
     for plugin_plan in plan.get_plugin_plans():
         inplan_repos = plugin_plan.get_repositories()
 
-        # filter by repo type
+        # filter by repo type and by the repos specified in a plan
         repos_to_consider = plan.type_to_repo_ids[plugin_plan.type]
-
-        # in case only certain repositories are specified in the migration plan
-        if inplan_repos:
-            repos_to_consider = set(inplan_repos).intersection(repos_to_consider)
+        repos_to_consider = set(inplan_repos).intersection(repos_to_consider)
 
         mongo_repo_q = mongo_Q(repo_id__in=repos_to_consider)
         mongo_repo_obj_ids = set(str(i.id) for i in Repository.objects(mongo_repo_q).only('id'))
@@ -641,11 +636,7 @@ def handle_outdated_resources(plan):
 
         # Mark removed or excluded importers
         inplan_imp_repos = plugin_plan.get_importers_repos()
-        if inplan_imp_repos:
-            mongo_imp_q = mongo_Q(repo_id__in=inplan_imp_repos)
-        else:
-            # simple migration plan case, all importers are migrated
-            mongo_imp_q = mongo_Q()
+        mongo_imp_q = mongo_Q(repo_id__in=inplan_imp_repos)
         mongo_imp_obj_ids = set(str(i.id) for i in Importer.objects(mongo_imp_q).only('id'))
         imp_types = plugin_plan.migrator.importer_migrators.keys()
 
@@ -655,11 +646,7 @@ def handle_outdated_resources(plan):
 
         # Mark removed or excluded distributors
         inplan_dist_repos = plugin_plan.get_distributors_repos()
-        if inplan_dist_repos:
-            mongo_dist_q = mongo_Q(repo_id__in=inplan_dist_repos)
-        else:
-            # simple migration plan case, all distributors are migrated
-            mongo_dist_q = mongo_Q()
+        mongo_dist_q = mongo_Q(repo_id__in=inplan_dist_repos)
         mongo_dist_obj_ids = set(str(i.id) for i in Distributor.objects(mongo_dist_q).only('id'))
         dist_types = plugin_plan.migrator.distributor_migrators.keys()
 
@@ -708,3 +695,12 @@ def handle_outdated_resources(plan):
         # Delete outdated distributions
         BaseDistribution.objects.filter(
             pulp2distributor__in=pulp2distributors_with_old_distributions_qs).delete()
+
+        # Remove relations to the pulp2repository in case the relation changed.
+        # Pulp2Distributors with is_migrated=false is handled and re-added properly at
+        # migration stage.
+        # NOTE: this needs to be removed last, the queries above use this relation.
+        not_migrated_dists = Pulp2Distributor.objects.filter(is_migrated=False).only('pulp_id')
+        Pulp2Distributor.pulp2_repos.through.objects.filter(
+            pulp2distributor__in=not_migrated_dists
+        ).delete()
